@@ -18,15 +18,9 @@ import torch.nn as nn
 import torch.optim as optim
 import torch.utils.data
 import torch.utils.data.distributed
-import torch.utils.tensorboard as tensorboard
 import torchvision.transforms as transforms
 from opacus import PrivacyEngine
-from opacus.layers import DifferentiallyPrivateDistributedDataParallel as DPDDP
-from opacus.utils import stats
-from opacus.utils.uniform_sampler import (
-    DistributedPoissonBatchSampler,
-    UniformWithReplacementSampler,
-)
+from opacus.distributed import DifferentiallyPrivateDistributedDataParallel as DPDDP
 from torch.nn.parallel import DistributedDataParallel as DDP
 from torchvision.datasets import CIFAR10
 from tqdm import tqdm
@@ -42,7 +36,6 @@ logger.setLevel(level=logging.INFO)
 
 
 def setup(args):
-
     if not torch.cuda.is_available():
         raise NotImplementedError(
             "DistributedDataParallel device_ids and output_device arguments \
@@ -124,7 +117,7 @@ def accuracy(preds, labels):
     return (preds == labels).mean()
 
 
-def train(args, model, train_loader, optimizer, epoch, device):
+def train(args, model, train_loader, optimizer, privacy_engine, epoch, device):
     start_time = datetime.now()
 
     model.train()
@@ -149,23 +142,20 @@ def train(args, model, train_loader, optimizer, epoch, device):
 
         losses.append(loss.item())
         top1_acc.append(acc1)
-        stats.update(stats.StatType.TRAIN, acc1=acc1)
 
         # compute gradient and do SGD step
         loss.backward()
 
         # make sure we take a step after processing the last mini-batch in the
         # epoch to ensure we start the next epoch with a clean state
-        if ((i + 1) % args.n_accumulation_steps == 0) or ((i + 1) == len(train_loader)):
-            optimizer.step()
-            optimizer.zero_grad()
-        else:
-            optimizer.virtual_step()
+        optimizer.step()
+        optimizer.zero_grad()
 
         if i % args.print_freq == 0:
             if not args.disable_dp:
-                epsilon, best_alpha = optimizer.privacy_engine.get_privacy_spent(
-                    args.delta
+                epsilon, best_alpha = privacy_engine.accountant.get_privacy_spent(
+                    delta=args.delta,
+                    alphas=[1 + x / 10.0 for x in range(1, 100)] + list(range(12, 64)),
                 )
                 print(
                     f"\tTrain Epoch: {epoch} \t"
@@ -204,7 +194,6 @@ def test(args, model, test_loader, device):
             top1_acc.append(acc1)
 
     top1_avg = np.mean(top1_acc)
-    stats.update(stats.StatType.TEST, acc1=top1_avg)
 
     print(f"\tTest set:" f"Loss: {np.mean(losses):.6f} " f"Acc@1: {top1_avg :.6f} ")
     return np.mean(top1_acc)
@@ -212,14 +201,13 @@ def test(args, model, test_loader, device):
 
 # flake8: noqa: C901
 def main():
-
     args = parse_args()
 
     if args.debug >= 1:
         logger.setLevel(level=logging.DEBUG)
 
     # Sets `world_size = 1` if you run on a single GPU with `args.local_rank = -1`
-    if args.device != "cpu":
+    if args.local_rank != -1 or args.device != "cpu":
         rank, local_rank, world_size = setup(args)
         device = local_rank
     else:
@@ -227,38 +215,6 @@ def main():
         rank = 0
         world_size = 1
 
-    if args.disable_dp and args.n_accumulation_steps > 1:
-        raise ValueError("Virtual steps only works with enabled DP")
-
-    if args.dist_algo == "ddp_hook" and not args.clip_per_layer:
-        raise ValueError(
-            "Please enable `--clip_per_layer` if you want to use Opacus DDP"
-        )
-
-    # The following few lines, enable stats gathering about the run
-    # 1. where the stats should be logged
-    stats.set_global_summary_writer(tensorboard.SummaryWriter(args.log_dir))
-    # 2. enable stats
-    stats.add(
-        # stats about gradient norms aggregated for all layers
-        stats.Stat(stats.StatType.GRAD, "AllLayers", frequency=0.1),
-        # stats about gradient norms per layer
-        stats.Stat(stats.StatType.GRAD, "PerLayer", frequency=0.1),
-        # stats about clipping
-        stats.Stat(stats.StatType.GRAD, "ClippingStats", frequency=0.1),
-        # stats on training accuracy
-        stats.Stat(stats.StatType.TRAIN, "accuracy", frequency=0.01),
-        # stats on validation accuracy
-        stats.Stat(stats.StatType.TEST, "accuracy"),
-    )
-
-    # The following lines enable stat gathering for the clipping process
-    # and set a default of per layer clipping for the Privacy Engine
-    clipping = {
-        "clip_per_layer": args.clip_per_layer,
-        "enable_stat": (rank == 0),
-    }
-
     if args.secure_rng:
         try:
             import torchcsprng as prng
@@ -292,25 +248,9 @@ def main():
         root=args.data_root, train=True, download=True, transform=train_transform
     )
 
-    if world_size > 1:
-        train_sampler = DistributedPoissonBatchSampler(
-            total_size=len(train_dataset),
-            sample_rate=args.sample_rate,
-            num_replicas=world_size,
-            rank=rank,
-            generator=generator,
-        )
-
-    else:
-        train_sampler = UniformWithReplacementSampler(
-            num_samples=len(train_dataset),
-            sample_rate=args.sample_rate,
-            generator=generator,
-        )
-
     train_loader = torch.utils.data.DataLoader(
         train_dataset,
-        batch_sampler=train_sampler,
+        batch_size=int(args.sample_rate * len(train_dataset)),
         generator=generator,
         num_workers=args.workers,
         pin_memory=True,
@@ -334,14 +274,10 @@ def main():
     # Use the right distributed module wrapper if distributed training is enabled
     if world_size > 1:
         if not args.disable_dp:
-            if args.dist_algo == "naive":
-                model = DPDDP(model)
-            elif args.dist_algo == "ddp_hook":
+            if args.clip_per_layer:
                 model = DDP(model, device_ids=[device])
             else:
-                raise NotImplementedError(
-                    f"Unrecognized argument for the distributed algorithm: {args.dist_algo}"
-                )
+                model = DPDDP(model)
         else:
             model = DDP(model, device_ids=[device])
 
@@ -359,6 +295,7 @@ def main():
     else:
         raise NotImplementedError("Optimizer not recognized. Please check spelling")
 
+    privacy_engine = None
     if not args.disable_dp:
         if args.clip_per_layer:
             # Each layer has the same clipping threshold. The total grad norm is still bounded by `args.max_per_sample_grad_norm`.
@@ -372,15 +309,17 @@ def main():
             max_grad_norm = args.max_per_sample_grad_norm
 
         privacy_engine = PrivacyEngine(
-            model,
-            sample_rate=args.sample_rate * args.n_accumulation_steps,
-            alphas=[1 + x / 10.0 for x in range(1, 100)] + list(range(12, 64)),
+            secure_mode=args.secure_rng,
+        )
+        clipping = "per_layer" if args.clip_per_layer else "flat"
+        model, optimizer, train_loader = privacy_engine.make_private(
+            module=model,
+            optimizer=optimizer,
+            data_loader=train_loader,
             noise_multiplier=args.sigma,
             max_grad_norm=max_grad_norm,
-            secure_rng=args.secure_rng,
-            **clipping,
+            clipping=clipping,
         )
-        privacy_engine.attach(optimizer)
 
     # Store some logs
     accuracy_per_epoch = []
@@ -392,7 +331,9 @@ def main():
             for param_group in optimizer.param_groups:
                 param_group["lr"] = lr
 
-        train_duration = train(args, model, train_loader, optimizer, epoch, device)
+        train_duration = train(
+            args, model, train_loader, optimizer, privacy_engine, epoch, device
+        )
         top1_acc = test(args, model, test_loader, device)
 
         # remember best acc@1 and save checkpoint
@@ -415,7 +356,6 @@ def main():
         )
 
     if rank == 0:
-
         time_per_epoch_seconds = [t.total_seconds() for t in time_per_epoch]
         avg_time_per_epoch = sum(time_per_epoch_seconds) / len(time_per_epoch_seconds)
         metrics = {
@@ -475,14 +415,6 @@ def parse_args():
         metavar="SR",
         help="sample rate used for batch construction (default: 0.005)",
     )
-    parser.add_argument(
-        "-na",
-        "--n_accumulation_steps",
-        default=1,
-        type=int,
-        metavar="N",
-        help="number of mini-batches to accumulate into an effective batch",
-    )
     parser.add_argument(
         "--lr",
         "--learning-rate",
@@ -555,7 +487,9 @@ def parse_args():
         "--secure-rng",
         action="store_true",
         default=False,
-        help="Enable Secure RNG to have trustworthy privacy guarantees. Comes at a performance cost",
+        help="Enable Secure RNG to have trustworthy privacy guarantees."
+        "Comes at a performance cost. Opacus will emit a warning if secure rng is off,"
+        "indicating that for production use it's recommender to turn it on.",
     )
     parser.add_argument(
         "--delta",
@@ -610,12 +544,6 @@ def parse_args():
         help="Choose the backend for torch distributed from: gloo, nccl, mpi",
     )
 
-    parser.add_argument(
-        "--dist_algo",
-        type=str,
-        default="naive",
-        help="Choose the algorithm for distributed DPSGD: `naive` for a simple aggregation with allreduce, or `ddp_hook` to use DDP with Opacus",
-    )
     parser.add_argument(
         "--clip_per_layer",
         action="store_true",
