@@ -14,7 +14,6 @@ import torch.nn as nn
 import torch.nn.functional as F
 import torch.optim as optim
 from opacus import PrivacyEngine
-from opacus.utils.uniform_sampler import UniformWithReplacementSampler
 from torchvision import datasets, transforms
 from tqdm import tqdm
 
@@ -47,7 +46,7 @@ class SampleConvNet(nn.Module):
         return "SampleConvNet"
 
 
-def train(args, model, device, train_loader, optimizer, epoch):
+def train(args, model, device, train_loader, optimizer, privacy_engine, epoch):
     model.train()
     criterion = nn.CrossEntropyLoss()
     losses = []
@@ -61,7 +60,9 @@ def train(args, model, device, train_loader, optimizer, epoch):
         losses.append(loss.item())
 
     if not args.disable_dp:
-        epsilon, best_alpha = optimizer.privacy_engine.get_privacy_spent(args.delta)
+        epsilon, best_alpha = privacy_engine.accountant.get_privacy_spent(
+            delta=args.delta
+        )
         print(
             f"Train Epoch: {epoch} \t"
             f"Loss: {np.mean(losses):.6f} "
@@ -71,7 +72,7 @@ def train(args, model, device, train_loader, optimizer, epoch):
         print(f"Train Epoch: {epoch} \t Loss: {np.mean(losses):.6f}")
 
 
-def test(args, model, device, test_loader):
+def test(model, device, test_loader):
     model.eval()
     criterion = nn.CrossEntropyLoss()
     test_loss = 0
@@ -101,21 +102,24 @@ def test(args, model, device, test_loader):
 
 def main():
     # Training settings
-    parser = argparse.ArgumentParser(description="PyTorch MNIST Example")
+    parser = argparse.ArgumentParser(
+        description="Opacus MNIST Example",
+        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
+    )
     parser.add_argument(
-        "-sr",
-        "--sample-rate",
-        type=float,
-        default=0.001,
-        metavar="SR",
-        help="sample rate used for batch construction (default: 0.001)",
+        "-b",
+        "--batch-size",
+        type=int,
+        default=64,
+        metavar="B",
+        help="Batch size",
     )
     parser.add_argument(
         "--test-batch-size",
         type=int,
         default=1024,
         metavar="TB",
-        help="input batch size for testing (default: 1024)",
+        help="input batch size for testing",
     )
     parser.add_argument(
         "-n",
@@ -123,7 +127,7 @@ def main():
         type=int,
         default=10,
         metavar="N",
-        help="number of epochs to train (default: 14)",
+        help="number of epochs to train",
     )
     parser.add_argument(
         "-r",
@@ -131,21 +135,21 @@ def main():
         type=int,
         default=1,
         metavar="R",
-        help="number of runs to average on (default: 1)",
+        help="number of runs to average on",
     )
     parser.add_argument(
         "--lr",
         type=float,
         default=0.1,
         metavar="LR",
-        help="learning rate (default: .1)",
+        help="learning rate",
     )
     parser.add_argument(
         "--sigma",
         type=float,
         default=1.0,
         metavar="S",
-        help="Noise multiplier (default 1.0)",
+        help="Noise multiplier",
     )
     parser.add_argument(
         "-c",
@@ -153,26 +157,26 @@ def main():
         type=float,
         default=1.0,
         metavar="C",
-        help="Clip per-sample gradients to this norm (default 1.0)",
+        help="Clip per-sample gradients to this norm",
     )
     parser.add_argument(
         "--delta",
         type=float,
         default=1e-5,
         metavar="D",
-        help="Target delta (default: 1e-5)",
+        help="Target delta",
     )
     parser.add_argument(
         "--device",
         type=str,
         default="cuda",
-        help="GPU ID for this process (default: 'cuda')",
+        help="GPU ID for this process",
     )
     parser.add_argument(
         "--save-model",
         action="store_true",
         default=False,
-        help="Save the trained model (default: false)",
+        help="Save the trained model",
     )
     parser.add_argument(
         "--disable-dp",
@@ -195,44 +199,21 @@ def main():
     args = parser.parse_args()
     device = torch.device(args.device)
 
-    kwargs = {"num_workers": 1, "pin_memory": True}
-
-    if args.secure_rng:
-        try:
-            import torchcsprng as prng
-        except ImportError as e:
-            msg = (
-                "To use secure RNG, you must install the torchcsprng package! "
-                "Check out the instructions here: https://github.com/pytorch/csprng#installation"
-            )
-            raise ImportError(msg) from e
-
-        generator = prng.create_random_device_generator("/dev/urandom")
-
-    else:
-        generator = None
-
-    train_dataset = datasets.MNIST(
-        args.data_root,
-        train=True,
-        download=True,
-        transform=transforms.Compose(
-            [
-                transforms.ToTensor(),
-                transforms.Normalize((MNIST_MEAN,), (MNIST_STD,)),
-            ]
-        ),
-    )
-
     train_loader = torch.utils.data.DataLoader(
-        train_dataset,
-        generator=generator,
-        batch_sampler=UniformWithReplacementSampler(
-            num_samples=len(train_dataset),
-            sample_rate=args.sample_rate,
-            generator=generator,
+        datasets.MNIST(
+            args.data_root,
+            train=True,
+            download=True,
+            transform=transforms.Compose(
+                [
+                    transforms.ToTensor(),
+                    transforms.Normalize((MNIST_MEAN,), (MNIST_STD,)),
+                ]
+            ),
         ),
-        **kwargs,
+        batch_size=args.batch_size,
+        num_workers=1,
+        pin_memory=True,
     )
     test_loader = torch.utils.data.DataLoader(
         datasets.MNIST(
@@ -247,26 +228,29 @@ def main():
         ),
         batch_size=args.test_batch_size,
         shuffle=True,
-        **kwargs,
+        num_workers=1,
+        pin_memory=True,
     )
     run_results = []
     for _ in range(args.n_runs):
         model = SampleConvNet().to(device)
 
         optimizer = optim.SGD(model.parameters(), lr=args.lr, momentum=0)
+        privacy_engine = None
+
         if not args.disable_dp:
-            privacy_engine = PrivacyEngine(
-                model,
-                sample_rate=args.sample_rate,
-                alphas=[1 + x / 10.0 for x in range(1, 100)] + list(range(12, 64)),
+            privacy_engine = PrivacyEngine(secure_mode=args.secure_rng)
+            model, optimizer, train_loader = privacy_engine.make_private(
+                module=model,
+                optimizer=optimizer,
+                data_loader=train_loader,
                 noise_multiplier=args.sigma,
                 max_grad_norm=args.max_per_sample_grad_norm,
-                secure_rng=args.secure_rng,
             )
-            privacy_engine.attach(optimizer)
+
         for epoch in range(1, args.epochs + 1):
-            train(args, model, device, train_loader, optimizer, epoch)
-        run_results.append(test(args, model, device, test_loader))
+            train(args, model, device, train_loader, optimizer, privacy_engine, epoch)
+        run_results.append(test(model, device, test_loader))
 
     if len(run_results) > 1:
         print(
@@ -276,8 +260,8 @@ def main():
         )
 
     repro_str = (
-        f"{model.name()}_{args.lr}_{args.sigma}_"
-        f"{args.max_per_sample_grad_norm}_{args.sample_rate}_{args.epochs}"
+        f"mnist_{args.lr}_{args.sigma}_"
+        f"{args.max_per_sample_grad_norm}_{args.batch_size}_{args.epochs}"
     )
     torch.save(run_results, f"run_results_{repro_str}.pt")
 
