@@ -2,23 +2,25 @@
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
 
+from typing import Dict
+
 import torch
+import torch.nn as nn
 from opacus.layers.dp_multihead_attention import SequenceBias
 
-from .utils import create_or_extend_grad_sample, register_grad_sampler
+from .utils import register_grad_sampler
 
 
 @register_grad_sampler(SequenceBias)
 def compute_sequence_bias_grad_sample(
-    layer: SequenceBias, A: torch.Tensor, B: torch.Tensor, batch_dim: int = 0
-) -> None:
+    layer: SequenceBias, activations: torch.Tensor, backprops: torch.Tensor
+) -> Dict[nn.Parameter, torch.Tensor]:
     """
     Computes per sample gradients for ``SequenceBias`` layer
 
     Args:
         layer: Layer
-        A: Activations
-        B: Backpropagations
-        batch_dim: Batch dimension position
+        activations: Activations
+        backprops: Backpropagations
     """
-    create_or_extend_grad_sample(layer.bias, B[:, -1], batch_dim)
+    return {layer.bias: backprops[:, -1]}
