@@ -11,7 +11,7 @@ from torch.functional import F
 
 
 def calc_sample_norms(
-    named_params: Iterator[Tuple[str, torch.Tensor]], flat: bool = True
+    named_params: Iterator[Tuple[str, torch.Tensor]], *, flat: bool = True
 ) -> List[torch.Tensor]:
     r"""
     Calculates the norm of the given tensors for each sample.
@@ -26,14 +26,15 @@ def calc_sample_norms(
         flat: A flag, when set to `True` returns a flat norm over all
             layers norms
 
+    Returns:
+        A list of tensor norms where length of the list is the number of layers
+
     Example:
         >>> t1 = torch.rand((2, 5))
         >>> t2 = torch.rand((2, 5))
-        >>> calc_sample_norms([("1", t1), ("2", t2)])
-            [tensor([1.5117, 1.0618])]
-
-    Returns:
-        A list of tensor norms where length of the list is the number of layers
+        >>> norms = calc_sample_norms([("1", t1), ("2", t2)])
+        >>> norms, norms[0].shape
+        ([tensor([...])], torch.Size([2]))
     """
     norms = [param.view(len(param), -1).norm(2, dim=-1) for name, param in named_params]
     # calc norm over all layer norms if flat = True
@@ -50,21 +51,20 @@ def calc_sample_norms_one_layer(param: torch.Tensor) -> torch.Tensor:
     assuming the each batch's dim is zero.
 
     It is equivalent to:
-    ```
-        calc_sample_norms(named_params=((None, param),))[0]
-    ```
+    `calc_sample_norms(named_params=((None, param),))[0]`
 
     Args:
         param: A tensor of shape ``[B, ...]`` where ``B``
             is the size of the batch and is the 0th dimension.
 
-    Example:
-        >>> t1 = torch.rand((2, 5))
-        >>> calc_sample_norms_one_layer(t1)
-            tensor([1.4757, 0.8128])
-
     Returns:
         A tensor of norms
+
+    Example:
+        >>> t1 = torch.rand((2, 5))
+        >>> norms = calc_sample_norms_one_layer(t1)
+        >>> norms, norms.shape
+        (tensor([...]), torch.Size([2]))
     """
     norms = param.view(len(param), -1).norm(2, dim=-1)
     return norms
@@ -84,13 +84,13 @@ def sum_over_all_but_batch_and_last_n(
         tensor: An input tensor of shape ``(B, ..., X[n_dims-1])``.
         n_dims: Number of dimensions to keep.
 
+    Returns:
+        A tensor of shape ``(B, ..., X[n_dims-1])``
+
     Example:
         >>> tensor = torch.ones(1, 2, 3, 4, 5)
         >>> sum_over_all_but_batch_and_last_n(tensor, n_dims=2).shape
         torch.Size([1, 4, 5])
-
-    Returns:
-        A tensor of shape ``(B, ..., X[n_dims-1])``
     """
     if tensor.dim() == n_dims + 1:
         return tensor
@@ -99,8 +99,43 @@ def sum_over_all_but_batch_and_last_n(
         return tensor.sum(dim=dims)
 
 
+def unfold2d(
+    input,
+    *,
+    kernel_size: Tuple[int, int],
+    padding: Tuple[int, int],
+    stride: Tuple[int, int],
+    dilation: Tuple[int, int],
+):
+    """
+    See :meth:`~torch.nn.functional.unfold`
+    """
+    *shape, H, W = input.shape
+    H_effective = (
+        H + 2 * padding[0] - (kernel_size[0] + (kernel_size[0] - 1) * (dilation[0] - 1))
+    ) // stride[0] + 1
+    W_effective = (
+        W + 2 * padding[1] - (kernel_size[1] + (kernel_size[1] - 1) * (dilation[1] - 1))
+    ) // stride[1] + 1
+    input = F.pad(input, (padding[0], padding[0], padding[1], padding[1]))
+    *shape_pad, H_pad, W_pad = input.shape
+    strides = list(input.stride())
+    strides = strides[:-2] + [
+        W_pad * dilation[0],
+        dilation[1],
+        W_pad * stride[0],
+        stride[1],
+    ]
+    out = input.as_strided(
+        shape + [kernel_size[0], kernel_size[1], H_effective, W_effective], strides
+    )
+
+    return out.reshape(input.size(0), -1, H_effective * W_effective)
+
+
 def unfold3d(
     tensor: torch.Tensor,
+    *,
     kernel_size: Union[int, Tuple[int, int, int]],
     padding: Union[int, Tuple[int, int, int]] = 0,
     stride: Union[int, Tuple[int, int, int]] = 1,
@@ -119,15 +154,15 @@ def unfold3d(
         stride: the stride of the sliding blocks in the input spatial dimensions
         dilation: the spacing between the kernel points.
 
-    Example:
-        >>> B, C, D, H, W = 3, 4, 5, 6, 7
-        >>> tensor = torch.arange(1,B*C*D*H*W+1.).view(B,C,D,H,W)
-        >>> unfold3d(tensor, kernel_size=2, padding=0, stride=1).shape
-        torch.Size([3, 32, 120])
-
     Returns:
         A tensor of shape ``(B, C * np.product(kernel_size), L)``, where L - output spatial dimensions.
         See :class:`torch.nn.Unfold` for more details
+
+    Example:
+        >>> B, C, D, H, W = 3, 4, 5, 6, 7
+        >>> tensor = torch.arange(1, B*C*D*H*W + 1.).view(B, C, D, H, W)
+        >>> unfold3d(tensor, kernel_size=2, padding=0, stride=1).shape
+        torch.Size([3, 32, 120])
     """
 
     if len(tensor.shape) != 5:
